from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define

from ..models.get_v1_jobs_response_200_jobs_item_state import GetV1JobsResponse200JobsItemState
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.get_v1_jobs_response_200_jobs_item_rubric_type_0 import (
        GetV1JobsResponse200JobsItemRubricType0,
    )


T = TypeVar("T", bound="GetV1JobsResponse200JobsItem")


@_attrs_define
class GetV1JobsResponse200JobsItem:
    """
    Attributes:
        id (str):
        task_type (str):
        state (GetV1JobsResponse200JobsItemState):
        payout_cents (int | None):
        deadline_seconds (int | None):
        created_at (str):
        expires_at (str):
        parent_job_id (None | str):
        parent_submission_id (None | str):
        rubric (GetV1JobsResponse200JobsItemRubricType0 | None | str | Unset):
    """

    id: str
    task_type: str
    state: GetV1JobsResponse200JobsItemState
    payout_cents: int | None
    deadline_seconds: int | None
    created_at: str
    expires_at: str
    parent_job_id: None | str
    parent_submission_id: None | str
    rubric: GetV1JobsResponse200JobsItemRubricType0 | None | str | Unset = UNSET

    def to_dict(self) -> dict[str, Any]:
        from ..models.get_v1_jobs_response_200_jobs_item_rubric_type_0 import (
            GetV1JobsResponse200JobsItemRubricType0,
        )

        id = self.id

        task_type = self.task_type

        state = self.state.value

        payout_cents: int | None
        payout_cents = self.payout_cents

        deadline_seconds: int | None
        deadline_seconds = self.deadline_seconds

        created_at = self.created_at

        expires_at = self.expires_at

        parent_job_id: None | str
        parent_job_id = self.parent_job_id

        parent_submission_id: None | str
        parent_submission_id = self.parent_submission_id

        rubric: dict[str, Any] | None | str | Unset
        if isinstance(self.rubric, Unset):
            rubric = UNSET
        elif isinstance(self.rubric, GetV1JobsResponse200JobsItemRubricType0):
            rubric = self.rubric.to_dict()
        else:
            rubric = self.rubric

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "id": id,
                "taskType": task_type,
                "state": state,
                "payoutCents": payout_cents,
                "deadlineSeconds": deadline_seconds,
                "createdAt": created_at,
                "expiresAt": expires_at,
                "parentJobId": parent_job_id,
                "parentSubmissionId": parent_submission_id,
            }
        )
        if rubric is not UNSET:
            field_dict["rubric"] = rubric

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.get_v1_jobs_response_200_jobs_item_rubric_type_0 import (
            GetV1JobsResponse200JobsItemRubricType0,
        )

        d = dict(src_dict)
        id = d.pop("id")

        task_type = d.pop("taskType")

        state = GetV1JobsResponse200JobsItemState(d.pop("state"))

        def _parse_payout_cents(data: object) -> int | None:
            if data is None:
                return data
            return cast(int | None, data)

        payout_cents = _parse_payout_cents(d.pop("payoutCents"))

        def _parse_deadline_seconds(data: object) -> int | None:
            if data is None:
                return data
            return cast(int | None, data)

        deadline_seconds = _parse_deadline_seconds(d.pop("deadlineSeconds"))

        created_at = d.pop("createdAt")

        expires_at = d.pop("expiresAt")

        def _parse_parent_job_id(data: object) -> None | str:
            if data is None:
                return data
            return cast(None | str, data)

        parent_job_id = _parse_parent_job_id(d.pop("parentJobId"))

        def _parse_parent_submission_id(data: object) -> None | str:
            if data is None:
                return data
            return cast(None | str, data)

        parent_submission_id = _parse_parent_submission_id(d.pop("parentSubmissionId"))

        def _parse_rubric(
            data: object,
        ) -> GetV1JobsResponse200JobsItemRubricType0 | None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                rubric_type_0 = GetV1JobsResponse200JobsItemRubricType0.from_dict(data)

                return rubric_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(GetV1JobsResponse200JobsItemRubricType0 | None | str | Unset, data)

        rubric = _parse_rubric(d.pop("rubric", UNSET))

        get_v1_jobs_response_200_jobs_item = cls(
            id=id,
            task_type=task_type,
            state=state,
            payout_cents=payout_cents,
            deadline_seconds=deadline_seconds,
            created_at=created_at,
            expires_at=expires_at,
            parent_job_id=parent_job_id,
            parent_submission_id=parent_submission_id,
            rubric=rubric,
        )

        return get_v1_jobs_response_200_jobs_item



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.get_v1_jobs_job_id_stages_response_200_stages_item import (
        GetV1JobsJobIdStagesResponse200StagesItem,
    )


T = TypeVar("T", bound="GetV1JobsJobIdStagesResponse200")


@_attrs_define
class GetV1JobsJobIdStagesResponse200:
    """
    Attributes:
        root_job_id (str):
        stages (list[GetV1JobsJobIdStagesResponse200StagesItem]):
    """

    root_job_id: str
    stages: list[GetV1JobsJobIdStagesResponse200StagesItem]

    def to_dict(self) -> dict[str, Any]:
        root_job_id = self.root_job_id

        stages = []
        for stages_item_data in self.stages:
            stages_item = stages_item_data.to_dict()
            stages.append(stages_item)

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "rootJobId": root_job_id,
                "stages": stages,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.get_v1_jobs_job_id_stages_response_200_stages_item import (
            GetV1JobsJobIdStagesResponse200StagesItem,
        )

        d = dict(src_dict)
        root_job_id = d.pop("rootJobId")

        stages = []
        _stages = d.pop("stages")
        for stages_item_data in _stages:
            stages_item = GetV1JobsJobIdStagesResponse200StagesItem.from_dict(stages_item_data)

            stages.append(stages_item)

        get_v1_jobs_job_id_stages_response_200 = cls(
            root_job_id=root_job_id,
            stages=stages,
        )

        return get_v1_jobs_job_id_stages_response_200



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.get_v1_public_jobs_job_id_response_200_verifier_context_acceptance_contract_must_include import (
        GetV1PublicJobsJobIdResponse200VerifierContextAcceptanceContractMustInclude,
    )
    from ..models.get_v1_public_jobs_job_id_response_200_verifier_context_acceptance_contract_output_schema import (
        GetV1PublicJobsJobIdResponse200VerifierContextAcceptanceContractOutputSchema,
    )


T = TypeVar("T", bound="GetV1PublicJobsJobIdResponse200VerifierContextAcceptanceContract")


@_attrs_define
class GetV1PublicJobsJobIdResponse200VerifierContextAcceptanceContract:
    """
    Attributes:
        max_bytes (int | Unset):
        must_include (GetV1PublicJobsJobIdResponse200VerifierContextAcceptanceContractMustInclude | Unset):
        output_schema (GetV1PublicJobsJobIdResponse200VerifierContextAcceptanceContractOutputSchema | Unset):
        deterministic_checks (list[str] | Unset):
    """

    max_bytes: int | Unset = UNSET
    must_include: (
        GetV1PublicJobsJobIdResponse200VerifierContextAcceptanceContractMustInclude | Unset
    ) = UNSET
    output_schema: (
        GetV1PublicJobsJobIdResponse200VerifierContextAcceptanceContractOutputSchema | Unset
    ) = UNSET
    deterministic_checks: list[str] | Unset = UNSET

    def to_dict(self) -> dict[str, Any]:
        max_bytes = self.max_bytes

        must_include: dict[str, Any] | Unset = UNSET
        if not isinstance(self.must_include, Unset):
            must_include = self.must_include.to_dict()

        output_schema: dict[str, Any] | Unset = UNSET
        if not isinstance(self.output_schema, Unset):
            output_schema = self.output_schema.to_dict()

        deterministic_checks: list[str] | Unset = UNSET
        if not isinstance(self.deterministic_checks, Unset):
            deterministic_checks = self.deterministic_checks

        field_dict: dict[str, Any] = {}

        field_dict.update({})
        if max_bytes is not UNSET:
            field_dict["maxBytes"] = max_bytes
        if must_include is not UNSET:
            field_dict["mustInclude"] = must_include
        if output_schema is not UNSET:
            field_dict["outputSchema"] = output_schema
        if deterministic_checks is not UNSET:
            field_dict["deterministicChecks"] = deterministic_checks

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.get_v1_public_jobs_job_id_response_200_verifier_context_acceptance_contract_must_include import (
            GetV1PublicJobsJobIdResponse200VerifierContextAcceptanceContractMustInclude,
        )
        from ..models.get_v1_public_jobs_job_id_response_200_verifier_context_acceptance_contract_output_schema import (
            GetV1PublicJobsJobIdResponse200VerifierContextAcceptanceContractOutputSchema,
        )

        d = dict(src_dict)
        max_bytes = d.pop("maxBytes", UNSET)

        _must_include = d.pop("mustInclude", UNSET)
        must_include: (
            GetV1PublicJobsJobIdResponse200VerifierContextAcceptanceContractMustInclude | Unset
        )
        if isinstance(_must_include, Unset):
            must_include = UNSET
        else:
            must_include = GetV1PublicJobsJobIdResponse200VerifierContextAcceptanceContractMustInclude.from_dict(
                _must_include
            )

        _output_schema = d.pop("outputSchema", UNSET)
        output_schema: (
            GetV1PublicJobsJobIdResponse200VerifierContextAcceptanceContractOutputSchema | Unset
        )
        if isinstance(_output_schema, Unset):
            output_schema = UNSET
        else:
            output_schema = GetV1PublicJobsJobIdResponse200VerifierContextAcceptanceContractOutputSchema.from_dict(
                _output_schema
            )

        deterministic_checks = cast(list[str], d.pop("deterministicChecks", UNSET))

        get_v1_public_jobs_job_id_response_200_verifier_context_acceptance_contract = cls(
            max_bytes=max_bytes,
            must_include=must_include,
            output_schema=output_schema,
            deterministic_checks=deterministic_checks,
        )

        return get_v1_public_jobs_job_id_response_200_verifier_context_acceptance_contract



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.get_v1_jobs_job_id_results_response_200_result import (
        GetV1JobsJobIdResultsResponse200Result,
    )


T = TypeVar("T", bound="GetV1JobsJobIdResultsResponse200")


@_attrs_define
class GetV1JobsJobIdResultsResponse200:
    """
    Attributes:
        result (GetV1JobsJobIdResultsResponse200Result):
    """

    result: GetV1JobsJobIdResultsResponse200Result

    def to_dict(self) -> dict[str, Any]:
        result = self.result.to_dict()

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "result": result,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.get_v1_jobs_job_id_results_response_200_result import (
            GetV1JobsJobIdResultsResponse200Result,
        )

        d = dict(src_dict)
        result = GetV1JobsJobIdResultsResponse200Result.from_dict(d.pop("result"))

        get_v1_jobs_job_id_results_response_200 = cls(
            result=result,
        )

        return get_v1_jobs_job_id_results_response_200



from __future__ import annotations

from typer.testing import CliRunner

from multipl_cli.app_state import AppState
from multipl_cli.commands import job
from multipl_cli.config import Config, Profile


def _state() -> AppState:
    config = Config(
        base_url="https://multipl.dev/api",
        active_profile="default",
        profiles={"default": Profile(name="default", poster_api_key="poster_test_key")},
    )
    return AppState(config=config, profile_name="default", base_url=config.base_url)


def test_job_help_lists_review_commands() -> None:
    runner = CliRunner()
    result = runner.invoke(job.app, ["--help"], obj=_state())
    assert result.exit_code == 0
    assert "accept" in result.stdout
    assert "reject" in result.stdout


def test_job_accept_invokes_review_helper(monkeypatch) -> None:
    calls: list[dict] = []

    def fake_review(state, *, job_id, decision, note, json_output):
        calls.append(
            {
                "state": state,
                "job_id": job_id,
                "decision": str(decision),
                "note": note,
                "json_output": json_output,
            }
        )

    monkeypatch.setattr(job, "_review_job", fake_review)

    runner = CliRunner()
    result = runner.invoke(
        job.app,
        ["accept", "job_123", "--note", "looks good", "--json"],
        obj=_state(),
    )
    assert result.exit_code == 0
    assert calls == [
        {
            "state": _state(),
            "job_id": "job_123",
            "decision": "accept",
            "note": "looks good",
            "json_output": True,
        }
    ]


def test_job_reject_invokes_review_helper(monkeypatch) -> None:
    calls: list[dict] = []

    def fake_review(state, *, job_id, decision, note, json_output):
        calls.append(
            {
                "state": state,
                "job_id": job_id,
                "decision": str(decision),
                "note": note,
                "json_output": json_output,
            }
        )

    monkeypatch.setattr(job, "_review_job", fake_review)

    runner = CliRunner()
    result = runner.invoke(job.app, ["reject", "job_456"], obj=_state())
    assert result.exit_code == 0
    assert calls == [
        {
            "state": _state(),
            "job_id": "job_456",
            "decision": "reject",
            "note": None,
            "json_output": False,
        }
    ]



from __future__ import annotations

from dataclasses import dataclass
from typing import Any

import httpx
from x402.http import PAYMENT_REQUIRED_HEADER, decode_payment_required_header
from x402.schemas import PaymentRequired, PaymentRequirements


@dataclass
class PaymentTerms:
    payment_required: PaymentRequired
    requirement: PaymentRequirements
    recipient: str
    amount: int | str
    asset: str
    network: str
    payment_context: str | None
    facilitator: str | None = None
    hint: str | None = None
    preview_json: Any | None = None
    commitment_sha256: str | None = None
    acceptance_report: Any | None = None
    metadata: dict[str, Any] | None = None


class PaymentTermsError(RuntimeError):
    pass


def _extract_payment_context(payment_required: PaymentRequired, payload: dict[str, Any]) -> str | None:
    value = payload.get("payment_context")
    if isinstance(value, str):
        return value
    extensions = payment_required.extensions or {}
    multipl = extensions.get("multipl") if isinstance(extensions, dict) else None
    info = multipl.get("info") if isinstance(multipl, dict) else None
    if isinstance(info, dict):
        ext_value = info.get("payment_context") or info.get("paymentContext")
        if isinstance(ext_value, str):
            return ext_value
    return None


def _select_requirement(payment_required: PaymentRequired) -> PaymentRequirements:
    for requirement in payment_required.accepts:
        if requirement.scheme == "exact":
            return requirement
    return payment_required.accepts[0]


def parse_payment_terms(response: httpx.Response) -> PaymentTerms:
    header_value = response.headers.get(PAYMENT_REQUIRED_HEADER) or response.headers.get(
        PAYMENT_REQUIRED_HEADER.lower()
    )
    if not header_value:
        raise PaymentTermsError("PAYMENT-REQUIRED header missing from 402 response")

    try:
        payment_required = decode_payment_required_header(header_value)
    except Exception as exc:
        raise PaymentTermsError("PAYMENT-REQUIRED header is invalid") from exc

    if not isinstance(payment_required, PaymentRequired):
        raise PaymentTermsError("unsupported x402 version")
    if not payment_required.accepts:
        raise PaymentTermsError("PAYMENT-REQUIRED missing accepted requirements")

    try:
        payload = response.json()
    except Exception:
        payload = {}
    if not isinstance(payload, dict):
        payload = {}

    requirement = _select_requirement(payment_required)
    payment_context = _extract_payment_context(payment_required, payload)

    recipient = str(payload.get("recipient") or requirement.pay_to)
    amount_value = payload.get("amount")
    if isinstance(amount_value, (int, float)):
        amount: int | str = int(amount_value)
    else:
        amount = requirement.amount

    asset = str(payload.get("asset") or requirement.asset)
    network = str(payload.get("network") or requirement.network)

    return PaymentTerms(
        payment_required=payment_required,
        requirement=requirement,
        recipient=recipient,
        amount=amount,
        asset=asset,
        network=network,
        payment_context=payment_context,
        facilitator=payload.get("facilitator"),
        hint=payload.get("hint"),
        preview_json=payload.get("previewJson"),
        commitment_sha256=payload.get("commitmentSha256"),
        acceptance_report=payload.get("acceptanceReport"),
        metadata=payload.get("metadata") if isinstance(payload.get("metadata"), dict) else None,
    )



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.post_v1_claims_claim_id_submit_response_200_job import (
        PostV1ClaimsClaimIdSubmitResponse200Job,
    )
    from ..models.post_v1_claims_claim_id_submit_response_200_submission import (
        PostV1ClaimsClaimIdSubmitResponse200Submission,
    )


T = TypeVar("T", bound="PostV1ClaimsClaimIdSubmitResponse200")


@_attrs_define
class PostV1ClaimsClaimIdSubmitResponse200:
    """
    Attributes:
        submission (PostV1ClaimsClaimIdSubmitResponse200Submission):
        job (PostV1ClaimsClaimIdSubmitResponse200Job):
    """

    submission: PostV1ClaimsClaimIdSubmitResponse200Submission
    job: PostV1ClaimsClaimIdSubmitResponse200Job

    def to_dict(self) -> dict[str, Any]:
        submission = self.submission.to_dict()

        job = self.job.to_dict()

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "submission": submission,
                "job": job,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.post_v1_claims_claim_id_submit_response_200_job import (
            PostV1ClaimsClaimIdSubmitResponse200Job,
        )
        from ..models.post_v1_claims_claim_id_submit_response_200_submission import (
            PostV1ClaimsClaimIdSubmitResponse200Submission,
        )

        d = dict(src_dict)
        submission = PostV1ClaimsClaimIdSubmitResponse200Submission.from_dict(d.pop("submission"))

        job = PostV1ClaimsClaimIdSubmitResponse200Job.from_dict(d.pop("job"))

        post_v1_claims_claim_id_submit_response_200 = cls(
            submission=submission,
            job=job,
        )

        return post_v1_claims_claim_id_submit_response_200



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define

from ..models.get_v1_workers_status_response_200_status import GetV1WorkersStatusResponse200Status
from ..types import UNSET, Unset

T = TypeVar("T", bound="GetV1WorkersStatusResponse200")


@_attrs_define
class GetV1WorkersStatusResponse200:
    """
    Attributes:
        status (GetV1WorkersStatusResponse200Status):
        claimed_by_poster_id (str | Unset):
    """

    status: GetV1WorkersStatusResponse200Status
    claimed_by_poster_id: str | Unset = UNSET

    def to_dict(self) -> dict[str, Any]:
        status = self.status.value

        claimed_by_poster_id = self.claimed_by_poster_id

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "status": status,
            }
        )
        if claimed_by_poster_id is not UNSET:
            field_dict["claimedByPosterId"] = claimed_by_poster_id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        status = GetV1WorkersStatusResponse200Status(d.pop("status"))

        claimed_by_poster_id = d.pop("claimedByPosterId", UNSET)

        get_v1_workers_status_response_200 = cls(
            status=status,
            claimed_by_poster_id=claimed_by_poster_id,
        )

        return get_v1_workers_status_response_200



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define

T = TypeVar("T", bound="GetV1PublicJobsJobIdResponse200Results")


@_attrs_define
class GetV1PublicJobsJobIdResponse200Results:
    """
    Attributes:
        is_submitted (bool):
        is_unlocked (bool):
        artifact_expires_at (None | str):
    """

    is_submitted: bool
    is_unlocked: bool
    artifact_expires_at: None | str

    def to_dict(self) -> dict[str, Any]:
        is_submitted = self.is_submitted

        is_unlocked = self.is_unlocked

        artifact_expires_at: None | str
        artifact_expires_at = self.artifact_expires_at

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "isSubmitted": is_submitted,
                "isUnlocked": is_unlocked,
                "artifactExpiresAt": artifact_expires_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        is_submitted = d.pop("isSubmitted")

        is_unlocked = d.pop("isUnlocked")

        def _parse_artifact_expires_at(data: object) -> None | str:
            if data is None:
                return data
            return cast(None | str, data)

        artifact_expires_at = _parse_artifact_expires_at(d.pop("artifactExpiresAt"))

        get_v1_public_jobs_job_id_response_200_results = cls(
            is_submitted=is_submitted,
            is_unlocked=is_unlocked,
            artifact_expires_at=artifact_expires_at,
        )

        return get_v1_public_jobs_job_id_response_200_results



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define

T = TypeVar("T", bound="PostV1WorkersClaimResponse200Worker")


@_attrs_define
class PostV1WorkersClaimResponse200Worker:
    """
    Attributes:
        id (str):
        name (str):
        is_claimed (bool):
        claimed_by_poster_id (None | str):
    """

    id: str
    name: str
    is_claimed: bool
    claimed_by_poster_id: None | str

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        name = self.name

        is_claimed = self.is_claimed

        claimed_by_poster_id: None | str
        claimed_by_poster_id = self.claimed_by_poster_id

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "id": id,
                "name": name,
                "isClaimed": is_claimed,
                "claimedByPosterId": claimed_by_poster_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id")

        name = d.pop("name")

        is_claimed = d.pop("isClaimed")

        def _parse_claimed_by_poster_id(data: object) -> None | str:
            if data is None:
                return data
            return cast(None | str, data)

        claimed_by_poster_id = _parse_claimed_by_poster_id(d.pop("claimedByPosterId"))

        post_v1_workers_claim_response_200_worker = cls(
            id=id,
            name=name,
            is_claimed=is_claimed,
            claimed_by_poster_id=claimed_by_poster_id,
        )

        return post_v1_workers_claim_response_200_worker



from __future__ import annotations

import json
from dataclasses import dataclass, field
from pathlib import Path
from typing import Any

from typer.testing import CliRunner

from multipl_cli.app_state import AppState
from multipl_cli.commands import job
from multipl_cli.config import Config, Profile


@dataclass
class _FakeHttpResponse:
    status_code: int
    payload: dict[str, Any]
    headers: dict[str, str] = field(default_factory=dict)
    content: bytes = b""

    def __post_init__(self) -> None:
        if not self.content:
            self.content = json.dumps(self.payload).encode("utf-8")

    def json(self) -> dict[str, Any]:
        return self.payload


@dataclass
class _FakeDetailedResponse:
    status_code: int
    parsed: Any = None
    headers: dict[str, str] = field(default_factory=dict)
    content: bytes = b"{}"


class _FakeHttpClient:
    def __init__(self, calls: list[dict[str, Any]], response: _FakeHttpResponse):
        self.calls = calls
        self.response = response

    def request(
        self,
        method: str,
        url: str,
        *,
        headers: dict[str, str] | None = None,
        json: dict[str, Any] | None = None,
    ) -> _FakeHttpResponse:
        self.calls.append(
            {
                "method": method,
                "url": url,
                "headers": headers,
                "json": json,
            }
        )
        return self.response


class _FakeClient:
    def __init__(self, http_client: _FakeHttpClient):
        self._http_client = http_client

    def get_httpx_client(self) -> _FakeHttpClient:
        return self._http_client


@dataclass
class _ParsedPayload:
    payload: dict[str, Any]

    def to_dict(self) -> dict[str, Any]:
        return self.payload


def _state() -> AppState:
    config = Config(
        base_url="https://multipl.dev/api",
        active_profile="default",
        profiles={
            "default": Profile(
                name="default",
                poster_api_key="poster_test_key",
                worker_api_key="worker_test_key",
            )
        },
    )
    return AppState(config=config, profile_name="default", base_url=config.base_url)


def _write_json(path: Path, payload: dict[str, Any]) -> Path:
    path.write_text(json.dumps(payload))
    return path


def test_job_create_legacy_wraps_input_payload(monkeypatch, tmp_path: Path) -> None:
    calls: list[dict[str, Any]] = []
    response = _FakeHttpResponse(
        status_code=201,
        payload={"job": {"id": "job_legacy", "state": "AVAILABLE"}},
    )
    fake_client = _FakeClient(_FakeHttpClient(calls, response))

    monkeypatch.setattr(job, "ensure_client_available", lambda: None)
    monkeypatch.setattr(job, "build_client", lambda _base_url: fake_client)
    monkeypatch.setattr(job, "request_with_x402", lambda request_fn, **_kwargs: request_fn(None))

    input_file = _write_json(tmp_path / "legacy-input.json", {"prompt": "hello"})
    runner = CliRunner()
    result = runner.invoke(
        job.app,
        [
            "create",
            "--task-type",
            "research.v1",
            "--input-file",
            str(input_file),
            "--idempotency-key",
            "idem-legacy",
            "--json",
        ],
        obj=_state(),
    )

    assert result.exit_code == 0
    assert len(calls) == 1
    sent_payload = calls[0]["json"]
    assert sent_payload["taskType"] == "research.v1"
    assert sent_payload["input"] == {"prompt": "hello"}
    assert "stages" not in sent_payload


def test_job_create_detects_full_request_and_preserves_top_level_stages(
    monkeypatch, tmp_path: Path
) -> None:
    calls: list[dict[str, Any]] = []
    response = _FakeHttpResponse(
        status_code=201,
        payload={"job": {"id": "job_staged", "state": "AVAILABLE"}},
    )
    fake_client = _FakeClient(_FakeHttpClient(calls, response))

    monkeypatch.setattr(job, "ensure_client_available", lambda: None)
    monkeypatch.setattr(job, "build_client", lambda _base_url: fake_client)
    monkeypatch.setattr(job, "request_with_x402", lambda request_fn, **_kwargs: request_fn(None))

    staged_request = {
        "taskType": "research.v1",
        "input": {"prompt": "root prompt"},
        "stages": [
            {"stageId": "plan", "name": "Plan", "taskType": "research.v1", "payoutCents": 50},
            {"stageId": "proof", "name": "Proof", "taskType": "research.v1", "payoutCents": 20},
        ],
    }
    input_file = _write_json(tmp_path / "staged-request.json", staged_request)
    runner = CliRunner()
    result = runner.invoke(
        job.app,
        [
            "create",
            "--input-file",
            str(input_file),
            "--idempotency-key",
            "idem-staged",
            "--json",
        ],
        obj=_state(),
    )

    assert result.exit_code == 0
    assert len(calls) == 1
    sent_payload = calls[0]["json"]
    assert sent_payload["taskType"] == "research.v1"
    assert "stages" in sent_payload
    assert sent_payload["stages"][0]["stageId"] == "plan"
    assert "stages" not in sent_payload["input"]


def test_job_create_request_file_mode_allows_task_type_override(monkeypatch, tmp_path: Path) -> None:
    calls: list[dict[str, Any]] = []
    response = _FakeHttpResponse(
        status_code=201,
        payload={"job": {"id": "job_override", "state": "AVAILABLE"}},
    )
    fake_client = _FakeClient(_FakeHttpClient(calls, response))

    monkeypatch.setattr(job, "ensure_client_available", lambda: None)
    monkeypatch.setattr(job, "build_client", lambda _base_url: fake_client)
    monkeypatch.setattr(job, "request_with_x402", lambda request_fn, **_kwargs: request_fn(None))

    request_file = _write_json(
        tmp_path / "request-file.json",
        {"taskType": "old.v1", "input": {"prompt": "hello"}},
    )
    runner = CliRunner()
    result = runner.invoke(
        job.app,
        [
            "create",
            "--request-file",
            "--task-type",
            "new.v1",
            "--input-file",
            str(request_file),
            "--idempotency-key",
            "idem-request-file",
            "--json",
        ],
        obj=_state(),
    )

    assert result.exit_code == 0
    assert len(calls) == 1
    assert calls[0]["json"]["taskType"] == "new.v1"


def test_job_stages_calls_endpoint_and_renders_pipeline(monkeypatch) -> None:
    monkeypatch.setattr(job, "ensure_client_available", lambda: None)
    monkeypatch.setattr(job, "build_client", lambda _base_url: object())

    payload = {
        "rootJobId": "root-stage-job",
        "stages": [
            {"stageIndex": 1, "stageId": "plan", "state": "AVAILABLE", "visibility": "GATED"},
            {"stageIndex": 2, "stageId": "proof", "state": "LOCKED", "visibility": "PUBLIC"},
        ],
    }

    def fake_get_job_stages(*, client, job_id, authorization):
        assert job_id == "job_123"
        assert authorization == "Bearer poster_test_key"
        return _FakeDetailedResponse(status_code=200, parsed=_ParsedPayload(payload))

    monkeypatch.setattr(job, "get_job_stages", fake_get_job_stages)

    runner = CliRunner()
    result = runner.invoke(job.app, ["stages", "job_123"], obj=_state())

    assert result.exit_code == 0
    assert "root-stage-job" in result.stdout
    assert "plan" in result.stdout
    assert "proof" in result.stdout



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="GetV1PublicJobsJobIdResponse200JobAcceptanceContractMustInclude")


@_attrs_define
class GetV1PublicJobsJobIdResponse200JobAcceptanceContractMustInclude:
    """
    Attributes:
        keys (list[str] | Unset):
        substrings (list[str] | Unset):
    """

    keys: list[str] | Unset = UNSET
    substrings: list[str] | Unset = UNSET

    def to_dict(self) -> dict[str, Any]:
        keys: list[str] | Unset = UNSET
        if not isinstance(self.keys, Unset):
            keys = self.keys

        substrings: list[str] | Unset = UNSET
        if not isinstance(self.substrings, Unset):
            substrings = self.substrings

        field_dict: dict[str, Any] = {}

        field_dict.update({})
        if keys is not UNSET:
            field_dict["keys"] = keys
        if substrings is not UNSET:
            field_dict["substrings"] = substrings

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        keys = cast(list[str], d.pop("keys", UNSET))

        substrings = cast(list[str], d.pop("substrings", UNSET))

        get_v1_public_jobs_job_id_response_200_job_acceptance_contract_must_include = cls(
            keys=keys,
            substrings=substrings,
        )

        return get_v1_public_jobs_job_id_response_200_job_acceptance_contract_must_include



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define

T = TypeVar("T", bound="GetV1JobsJobIdResultsResponse402Metadata")


@_attrs_define
class GetV1JobsJobIdResultsResponse402Metadata:
    """
    Attributes:
        job_id (str):
        task_type (str):
        submitted_at (str):
    """

    job_id: str
    task_type: str
    submitted_at: str

    def to_dict(self) -> dict[str, Any]:
        job_id = self.job_id

        task_type = self.task_type

        submitted_at = self.submitted_at

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "jobId": job_id,
                "taskType": task_type,
                "submittedAt": submitted_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        job_id = d.pop("jobId")

        task_type = d.pop("taskType")

        submitted_at = d.pop("submittedAt")

        get_v1_jobs_job_id_results_response_402_metadata = cls(
            job_id=job_id,
            task_type=task_type,
            submitted_at=submitted_at,
        )

        return get_v1_jobs_job_id_results_response_402_metadata
